import 'package:flutter/material.dart';
import 'package:charts_flutter/flutter.dart' as charts;
import 'package:material_segmented_control/material_segmented_control.dart';
import '../cryfetch.dart';
import '../models/cryptocurrency.dart';
import '../services/chart_fetcher.dart';
import './chart_coin_picker.dart';

class Chart extends StatefulWidget {
  final bool animate;

  const Chart({Key? key, this.animate = true}) : super(key: key);

  @override
  State<Chart> createState() => _ChartState();
}

class _ChartState extends State<Chart> {
  final Map<int, Widget> _options = {
    7: const Text('WEEK'),
    30: const Text('MONTH'),
    365: const Text('YEAR'),
    -1: const Text('ALL TIME'),
  };

  int _currentSelection = 7;

  List<Cryptocurrency> selectedCryptos = [];

  void _navigateAndDisplayCoins(BuildContext context) async {
    final Cryptocurrency? selectedCoin = await Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => ChartCoinPicker(
          CryFetch.cryptoList
              .where((crypto) => !selectedCryptos.contains(crypto))
              .toList(),
        ),
      ),
    );

    if (selectedCoin != null) {
      setState(() {
        selectedCryptos.add(selectedCoin);
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return !CryFetch.hasNetwork
        ? Center(
            child: const Text(
              "Grafovi nisu dostupni\nbez internetske veze.",
              style: TextStyle(
                fontSize: 20,
              ),
            ),
          )
        : Center(
            child: selectedCryptos.isNotEmpty
                ? Column(
                    children: [
                      (MaterialSegmentedControl(
                        children: _options,
                        selectionIndex: _currentSelection,
                        borderColor: Colors.grey,
                        selectedColor: Colors.redAccent,
                        unselectedColor: Colors.white,
                        borderRadius: 16.0,
                        onSegmentChosen: (int index) {
                          setState(() {
                            _currentSelection = index;
                          });
                        },
                      )),
                      (FutureBuilder<
                          List<charts.Series<CryptoPrice, DateTime>>>(
                        future: ChartFetcher.fetchChartData(
                            selectedCryptos,
                            _currentSelection > 0
                                ? _currentSelection.toString()
                                : "max"),
                        builder: (BuildContext context,
                            AsyncSnapshot<
                                    List<charts.Series<CryptoPrice, DateTime>>>
                                snapshot) {
                          if (!snapshot.hasData) {
                            return const CircularProgressIndicator();
                          } else {
                            return Expanded(
                              child: charts.TimeSeriesChart(
                                snapshot.data!,
                                animate: widget.animate,
                                dateTimeFactory:
                                    const charts.LocalDateTimeFactory(),
                              ),
                            );
                          }
                        },
                      )),
                      ConstrainedBox(
                        constraints: const BoxConstraints(maxHeight: 175),
                        child: ListView.builder(
                          itemBuilder: (context, index) {
                            final Cryptocurrency crypto =
                                selectedCryptos[index];
                            return ListTile(
                              title: Text(crypto.name),
                              subtitle: Text(crypto.currentPrice.toString() +
                                  Cryptocurrency.currencySymbol),
                              leading: IconButton(
                                icon: Icon(Icons.circle,
                                    color:
                                        ChartFetcher.cryptoColorFromId(index)),
                                onPressed: () {
                                  setState(() {
                                    selectedCryptos.remove(crypto);
                                  });
                                },
                              ),
                              trailing: IconButton(
                                icon:
                                    const Icon(Icons.delete, color: Colors.red),
                                onPressed: () {
                                  setState(() {
                                    selectedCryptos.remove(crypto);
                                  });
                                },
                              ),
                            );
                          },
                          itemCount: selectedCryptos.length,
                          scrollDirection: Axis.vertical,
                          shrinkWrap: true,
                        ),
                      ),
                      selectedCryptos.length == 5
                          ? const Padding(
                              padding: EdgeInsets.all(10.0),
                              child: Text(
                                  "Only 5 cryptos can be displayed at a time"),
                            )
                          : (ElevatedButton(
                              onPressed: () =>
                                  _navigateAndDisplayCoins(context),
                              child: const Padding(
                                padding: EdgeInsets.zero,
                                child: Text(
                                  'Add Crypto',
                                  style: TextStyle(
                                    fontSize: 14,
                                    fontWeight: FontWeight.w700,
                                    color: Colors.white,
                                  ),
                                ),
                              ),
                            )),
                    ],
                  )
                : ElevatedButton(
                    onPressed: () => _navigateAndDisplayCoins(context),
                    child: Padding(
                      padding: EdgeInsets.zero,
                      child: Row(
                        children: const <Widget>[
                          Text(
                            'Add cryptocurrency to start',
                            style: TextStyle(
                              fontSize: 20,
                              fontWeight: FontWeight.w700,
                              color: Colors.white,
                            ),
                          ),
                        ],
                        mainAxisSize: MainAxisSize.min,
                        mainAxisAlignment: MainAxisAlignment.spaceAround,
                      ),
                    ),
                  ),
          );
  }
}
